#![feature(proc_macro_hygiene, decl_macro)]
use failure::Error;
use oauth2::{AuthUrl, ClientId, ClientSecret, TokenResponse, TokenUrl};
use oauth2::basic::BasicClient;
use oauth2::reqwest::http_client;
use reqwest::{header::{AUTHORIZATION, HeaderMap, HeaderValue}};
use serde::{Serialize, Deserialize};
use rocket_contrib::json::Json;
use rocket_cors::{AllowedOrigins, CorsOptions};
use rocket::{http::Method,config::{Config, Environment}};
#[macro_use] extern crate rocket;

// structs for holding ticket data, easy conversion to JSON is needed for later submission to the LibAnswers API
#[derive(Serialize, Deserialize, Debug)]
struct TicketStruct {
    iid: i32,
    formId: i32,
    ownerId: i32,
    name: String,
    description: String,
    instance_id: i32,
    fields: Vec<FieldStruct>
}
#[derive(Serialize, Deserialize, Debug)]
struct FieldStruct {
    field_id: i32,
    label: String,
    val: String
}

// takes the information from the rocket server, puts the data into a JSON struct, and calls the function to send a ticket (with JSON data payload) to LibAnswers
#[post("/", format = "json", data = "<ticket>")]
fn handle_post(ticket: Json<TicketStruct>) {
    create_ticket(ticket).unwrap();
}

// rocket server function. makes listening for data on a port quite simple and handles CORS requests with minimal headache
fn listen_for_post() {
    let config = Config::build(Environment::Production)
        .address("127.0.0.1")
        .port(8080)
        .finalize().unwrap();
    let cors = CorsOptions::default()
        .allowed_origins(AllowedOrigins::all())
        .allowed_methods(
            vec![Method::Post]
                .into_iter()
                .map(From::from)
                .collect(),
        )
        .allow_credentials(true);
    rocket::custom(config)
        .mount("/", routes![handle_post])
        .attach(cors.to_cors().unwrap())
        .launch();
}

// splitting and formatting of the incoming JSON ticket struct to something slightly more appropriate for the submission function to work with
fn handle_fields(ticket: Json<TicketStruct>) -> (String, String, Json<TicketStruct>) {
    let mut title_question = "".to_owned();
    let mut question_body = "".to_owned();
    let space = " ".to_owned();
    let breake = "\n".to_owned();
    title_question.push_str(&ticket.fields[0].val.to_string());
    title_question.push_str(&space);
    for field in &ticket.fields {
        let field: &FieldStruct = field;
        if field.val == "" {
            continue;
        } else if field.label.contains("operating") && !field.val.eq("") {
            title_question.push_str(&field.val.to_string());
            title_question.push_str(&space);
        } else if field.label.contains("browser") && !field.val.eq("") {
            title_question.push_str(&field.val.to_string());
            title_question.push_str(&space);
        } else if field.label.contains("Website update") && !field.val.eq("") {
            question_body.push_str("Website update: ");
            question_body.push_str(&field.val.to_string());
            question_body.push_str(&space);
            question_body.push_str(&breake);
        } else if field.label.contains("rotator") && !field.val.eq("") {
            question_body.push_str("Rotator: ");
            question_body.push_str(&field.val.to_string());
            question_body.push_str(&space);
            question_body.push_str(&breake);
        } else if field.label.contains("When should this be published?") && !field.val.eq("") {
            question_body.push_str("Publish on: ");
            question_body.push_str(&field.val.to_string());
            question_body.push_str(&space);
            question_body.push_str(&breake);
        } else if field.label.contains("When should it be unpublished?") && !field.val.eq("") {
            question_body.push_str("Unpublish on: ");
            question_body.push_str(&field.val.to_string());
            question_body.push_str(&space);
            question_body.push_str(&breake);
        } else if field.label.contains("Describe the nature of your problem") && !field.val.eq("") {
            question_body.push_str(&field.val.to_string());
            question_body.push_str(&space);
            question_body.push_str(&breake);
        } else if field.label.contains("Include a title, description, and link.") && !field.val.eq("") {
            question_body.push_str(&field.val.to_string());
            question_body.push_str(&space);
            question_body.push_str(&breake);
        }
        // The following piece will fail when a screenshot is added to the question form. this is an issue on the Springshare side, as their code does not upload the image to AWS in time for them to return a nice URL
        // else if field.label.contains("Screenshots") && !field.val.eq("") {
        //     question_body.push_str("Screenshot(s): ");
        //     // question_body.push_str(&field.val.to_string());
        //     question_body.push_str(&space);
        //     question_body.push_str(&breake);
        // }
    }
    return (title_question, question_body, ticket);
}
#[tokio::main]
async fn create_ticket(ticket: Json<TicketStruct>) -> Result<(), Error> {
    // client needed for oauth 2.0, ClientSecret is a token that will need to be replaced
    let client =
    BasicClient::new(
        ClientId::new(<id>.to_string()),
        Some(ClientSecret::new(<secret>.to_string())),
        AuthUrl::new("https://uis.libanswers.com/api/1.1/ticket/create".to_string())?,
        Some(TokenUrl::new("https://uis.libanswers.com/api/1.1/oauth/token".to_string())?),
    );
    let token_result = client
    .exchange_client_credentials()
    .request(http_client)?;
    let (title_question, question_body, ticket) = handle_fields(ticket);
    let quid = "5231"; // ticket queue ID, determines where the ticket will go
    let pquest = title_question;
    let pdetails = question_body;
    let pname = ticket.fields[9].val.to_string();
    let pemail = ticket.fields[10].val.to_string();
    let ticket_form = reqwest::multipart::Form::new()
        .text("quid", quid)
        .text("pquestion", pquest)
        .text("pdetails", pdetails)
        .text("pname", pname)
        .text("pemail", pemail);
    let bearer: &str = "Bearer ";
    let token_val: &str = token_result
                                .access_token()
                                .secret();
    let header_string: String = format!("{}{}", bearer, token_val);
    let mut headers = HeaderMap::new();
    let header_val = HeaderValue::from_str(&header_string);
    headers.insert(AUTHORIZATION, header_val?);
    let client2 = reqwest::Client::new();
    let _res = client2.post("https://uis.libanswers.com/api/1.1/ticket/create")
                    .multipart(ticket_form)
                    .headers(headers)
                    .send().await;
    Ok(())
}
fn main() {
    listen_for_post();
}